# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""User proxy agent model."""

from .user_proxy import WaldiezUserProxy
from .user_proxy_data import WaldiezUserProxyData

__all__ = ["WaldiezUserProxy", "WaldiezUserProxyData"]
